##
## IBM Confidential
## © Copyright IBM Corp. 2020
## The source code for this program is not published or otherwise divested of its trade secrets,
## irrespective of what has been deposited with the U.S. Copyright Office.
## =============================================================================================
##

$INSTALL_DIR = "C:\Program Files\IBM\Metrics Collector for SAP"
$DOWNLOADS_DIR = "$INSTALL_DIR\Downloads"
$UPDATE_TASK = "IBM Metric Collector for SAP Updator"
$ErrorActionPreference='Stop'

# Create the Metric Collector for SAP Director to store all the files.
Function Create-Directory {
    PARAM ([string]$DirPath)
    try {
    if (Test-Path "$DirPath") {
        Write-host "We detected already installed IBM Metric Collector for SAP, would you like to uninstall? (Default is No)" -ForegroundColor Yellow 
        $Readhost = Read-Host " ( y / n ) " 
        
        Switch ($ReadHost) 
            { 
                Y {
                    Write-host "Continuing installing metric collector..."
                    if (Test-Path $DirPath\sap-metrics-collector.exe){
                    & $DirPath\sap-metrics-collector.exe --service stop
                    Start-Sleep -s 1
                    & $DirPath\sap-metrics-collector.exe --service uninstall
                    Start-Sleep -s 1
                }
                    Remove-Item -Recurse "$DirPath" -ErrorAction Stop
                } 
                N {
                    Write-Host "Exiting Metric Collector Installation."
                    Start-Sleep -s 1
                    Exit
                  } 
                Default { 
                    Write-Host "Exiting Metric Collector Installation."
                    Start-Sleep -s 1
                    Exit
                } 
            }
    }

    Write-Host "Creating new folder - $DirPath"
    New-Item -ItemType "Directory" -Path $DirPath

  } Catch {
    Write-Host $_.Exception|format-list -force | Out-String 
    Write-Host "Unable to Install Metric Collector." -ForegroundColor red
    Start-Sleep -s 3
    Exit                             
} 
}

# Create Schedule Task to start the Updator Script.
function Add-Scheduled-Task {
    PARAM ([string] $TaskName, [string] $ScriptName)
    if ($(Get-ScheduledTask $TaskName -ErrorAction Ignore).TaskName) {
       Write-Host "Scheduled task already exists: $TaskName"
       return
    }

    Write-Host "Adding scheduled task: $TaskName"
    $action = New-ScheduledTaskAction  `
      -Execute "Powershell.exe"  `
      -Argument "-File `"$INSTALL_DIR\$ScriptName`" -WindowStyle Hidden"  `
      -WorkingDirectory "$INSTALL_DIR"
    $trigger = New-ScheduledTaskTrigger `
        -Daily `
        -At 3am
    Register-ScheduledTask -Action $action -Trigger $trigger  `
      -TaskName "$TaskName"  `
      -Description "$TaskName" -User "System"
    Write-Host "Added scheduled task: $TaskName"
  }

try {

    Write-Host "Welcome to IBM Metric Collector for SAP"
    # Find the location where the install script is running from
    $ScriptDir = Split-Path $script:MyInvocation.MyCommand.Path
    Create-Directory $INSTALL_DIR
    Write-Host ""
    Write-Host "------------------"
    Write-Host ""
    $API_Key = Read-Host "Please Enter the API Key received from IBM"

    #Copy all the files thats found the install scipt directory
    Copy-Item "$ScriptDir\telegraf-windows.conf.template" -Destination "$INSTALL_DIR\telegraf.conf"
    Copy-Item "$ScriptDir\sap.xml" -Destination "$INSTALL_DIR"
    Copy-Item "$ScriptDir\sap-metrics-collector.exe" -Destination "$INSTALL_DIR"
    Copy-Item "$ScriptDir\update-metric-collector.ps1" -Destination "$INSTALL_DIR"
    Copy-Item "$ScriptDir\version" -Destination "$INSTALL_DIR"
    $content = Get-Content -Path "$INSTALL_DIR\telegraf.conf"
    $newcontent = $content -replace '<your IBM api key here>', "$API_Key"
    $newcontent | Set-Content -Path "$INSTALL_DIR\telegraf.conf"

    & $INSTALL_DIR\sap-metrics-collector.exe --service install --config $INSTALL_DIR\telegraf.conf
    Write-Host "Starting SAP Metric Collector ..."
    & $INSTALL_DIR\sap-metrics-collector.exe --service start
    Write-Host "Sucessfully Installed IBM Metric Collector for SAP." -ForegroundColor green 
    Write-Host "Please wait 1 min before trying to view metrics: <http://localhost:18181/sap/metrics>"
    Start-Sleep -s 2

    # Start the Updator Task to run everyday at 3am.
    Add-Scheduled-Task $UPDATE_TASK update-metric-collector.ps1

} Catch {
    Write-Host "Unable to Install Metric Collector." -ForegroundColor red  
}