##
## IBM Confidential
## © Copyright IBM Corp. 2020
## The source code for this program is not published or otherwise divested of its trade secrets,
## irrespective of what has been deposited with the U.S. Copyright Office.
## =============================================================================================
##

$INSTALL_DIR = "C:\Program Files\IBM\Metrics Collector for SAP"
$WINDOWS_URL = "ftp://public.dhe.ibm.com/cloud/bluemix/sap/windows/"
$DOWNLOAD_DIR = "C:\Program Files\IBM\Download"
$TEMP_FILE = "C:\Program Files\IBM\Download\sap-metrics-collector-v1.0.zip"
$LOG_FILE = "C:\Program Files\IBM\Metrics Collector for SAP\sap.log"

#Function to get all files
function Get-FtpDir
{
    PARAM ([string]$url)
    $request = [Net.WebRequest]::Create($url)
    $request.Method = [System.Net.WebRequestMethods+FTP]::ListDirectory
    $response = $request.GetResponse()
    $reader = New-Object IO.StreamReader $response.GetResponseStream()
    $readline = $reader.ReadLine()
    $output = New-Object System.Collections.Generic.List[System.Object]
    while ($readline -ne $null)
    {
        $output.Add($readline)
        $readline = $reader.ReadLine()
    }
    $reader.Close()
    $response.Close()
    $output
}

# Open the version file and parse the version number
function Get-Parsed-Version {
    PARAM ([string]$filename)
    $ver1 = $filename -replace 'sap-metrics-collector-v', ""
    $version = $ver1 -replace '.zip', ""
    #$version = $ver1 -replace '.tar.gz', ""
    $version
}

# Check the log size, and if its bigger than 1MB then rotate log.
function Execute-Log-Rotation {
    $logFileSize = $(Get-Item "$LOG_FILE" -ErrorAction Ignore).Length/1kb
    if ($logFileSize -ge 1024) {
      Move-Item -Force "$LOG_FILE" "$LOG_FILE.1"
    }
}

# Look for the latest SAP Metric Collector, if found update. 
function Get-Latest-App {
    PARAM ([string]$latest_url, [string]$file)
    Write-Output $latest_url
    $fileuri = New-Object System.Uri($latest_url)
    $webclient = New-Object System.Net.WebClient
    if(!(Test-Path -path $DOWNLOAD_DIR))  
    {  
        New-Item -ItemType directory -Path $DOWNLOAD_DIR       
    }
    $webclient.DownloadFile($fileuri, "$DOWNLOAD_DIR\$file")

    if(Test-Path -path "$DOWNLOAD_DIR\sap") {
        Remove-Item -Recurse -Force "$DOWNLOAD_DIR\sap"
    }

    #Expand-Archive -LiteralPath "$DOWNLOAD_DIR\$file" -DestinationPath $DOWNLOAD_DIR
    Expand-Archive -LiteralPath $TEMP_FILE -DestinationPath $DOWNLOAD_DIR
    
    if (Test-Path -path $INSTALL_DIR\sap-metrics-collector.exe) {
        & $INSTALL_DIR\sap-metrics-collector.exe --service stop
        Start-Sleep -s 1
        & $INSTALL_DIR\sap-metrics-collector.exe --service uninstall
        Start-Sleep -s 1
    } else {
        & $DOWNLOAD_DIR\sap\sap-metrics-collector.exe --service stop
        Start-Sleep -s 1
        & $DOWNLOAD_DIR\sap\sap-metrics-collector.exe --service uninstall
        Start-Sleep -s 1
    }
    Move-Item "$DOWNLOAD_DIR\sap\sap-metrics-collector.exe" "$INSTALL_DIR\sap-metrics-collector.exe" -Force
    Move-Item "$DOWNLOAD_DIR\sap\version" "$INSTALL_DIR\version" -Force
    if (!(Test-Path -path $INSTALL_DIR\sap.xml)) {
        Move-Item "$DOWNLOAD_DIR\sap\sap.xml" "$INSTALL_DIR\sap.xml" -Force
    }
    if (!(Test-Path -path $INSTALL_DIR\telegraf.conf)) {
        Move-Item "$DOWNLOAD_DIR\sap\telegraf-windows.conf.template" "$INSTALL_DIR\telegraf.conf" -Force
    }
}

Execute-Log-Rotation
$uri = New-Object System.Uri("$WINDOWS_URL")

# Check for current version, if it's not found, then automatically install the latest version.
try {
    $curr_version_output = Get-Content "$INSTALL_DIR\version" -First 1
} catch {
    $files = Get-FTPDir $uri
    foreach ($file in $files)
    {
        if ($file -like "sap-metrics-collector-*.tar.gz")
        {
            Get-Latest-App $WINDOWS_URL$file
            Move-Item "$DOWNLOAD_DIR\sap\sap.xml" "$INSTALL_DIR\sap.xml" -Force
            Move-Item "$DOWNLOAD_DIR\sap\telegraf-windows.conf.template" "$INSTALL_DIR\template.conf" -Force
            & $INSTALL_DIR\sap-metrics-collector.exe --service install --config $INSTALL_DIR\telegraf.conf
            & $INSTALL_DIR\sap-metrics-collector.exe --service start
            break
        }
    }
    exit
}

# Compare the latest version vs. installed version, if new version found, then install.
$curr_version = Get-Parsed-Version $curr_version_output
$files = Get-FTPDir $uri
    foreach ($file in $files)
{
    if ($file -like "sap-metrics-collector-*.tar.gz")
    {
        $version = Get-Parsed-Version $file
        Write-Output $version
        if ($version -gt $curr_version) {
            Get-Latest-App "$WINDOWS_URL$file" "$file"
            & $INSTALL_DIR\sap-metrics-collector.exe --service install --config $INSTALL_DIR\telegraf.conf
            & $INSTALL_DIR\sap-metrics-collector.exe --service start
            break
        } 
    }
}
